==> photoprint_0.3.3.orig.tar.gz: photoprint-0.3.3/layout_single.cpp <==
/*
 * layout_single.cpp - Has responsibility for tracking the list of images and their layout.
 * Also has responsibility for building the ImageSource stack at print time.
 *
 * Copyright (c) 2004 by Alastair M. Robinson
 * Distributed under the terms of the GNU General Public License -
 * see the file named "COPYING" for more details.
 *
 * 2004-12-24: Now opens the imagesource when placing an image, since we need both the
 *             natural size of the image (which the thumbnail code could provide)
 *             and the resolution of the image (which the thumbnail code can't provide).
 *             Getting the actual imagesource (with CMS transform) from the ImageInfo
 *             is now delegated to the superclasses.
 *
 * 2005-08-21: Added allowcropping and allowrotation parameters to allow for the menu
 *             generalisation.  FIXME: Make sure rotation behaves sanely.
 */

#include <iostream>
#include <string.h>

#include "support/generaldialogs.h"
#include "dialogs.h"
#include "pixbufthumbnail/egg-pixbuf-thumbnail.h"
#include "support/rotatepixbuf.h"
#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_crop.h"
#include "imagesource/imagesource_rotate.h"
#include "photoprint_state.h"
#include "pp_layout_single.h"

#include "layout_single.h"

using namespace std;


ConfigTemplate Layout_SingleDB::Template[]=
{
	ConfigTemplate("LeftMargin",10),
	ConfigTemplate("RightMargin",10),
	ConfigTemplate("TopMargin",10),
	ConfigTemplate("BottomMargin",10),
	ConfigTemplate()
};


void Layout_Single_ImageInfo::Init()
{
	Layout_Single *l=(Layout_Single *)&layout;
	ImageSource *is=ISLoadImage(filename);
	image_width=is->width;
	image_height=is->height;
	native_xres=is->xres;
	native_yres=is->yres;
	delete is;
	
	l->GetImageableArea();
	int sw=int((image_width*72.0)/native_xres);
	int sh=int((image_height*72.0)/native_yres);
	int mm=(l->imageablewidth>l->imageableheight);
	mm^=image_width>image_height;
	if(mm)
	{
		rotation=PP_ROTATION_90;
		sw=int((image_height*72.0)/native_yres);
		sh=int((image_width*72.0)/native_xres);
	}
	else
		rotation=PP_ROTATION_NONE;
	int s=100;
	int s2=100;
	if(sw>l->imageablewidth)
		s=(100*l->imageablewidth)/sw;
	if(sh>l->imageableheight)
		s2=(100*l->imageableheight)/sh;

	if(s2<s)
		s=s2;
	
	hscale=s;
	vscale=s;
}


Layout_Single_ImageInfo::Layout_Single_ImageInfo(Layout_Single &layout, const char *filename,int page,bool allowcropping,PP_ROTATION rotation)
	: Layout_ImageInfo((Layout &)layout,filename,page,allowcropping,rotation), hscale(100), vscale(100)
{
	Init();
}


Layout_Single_ImageInfo::Layout_Single_ImageInfo(Layout_Single &layout, Layout_ImageInfo *ii,int page,bool allowcropping,PP_ROTATION rotation)
	: Layout_ImageInfo((Layout &)layout,ii,page,allowcropping,rotation), hscale(100), vscale(100)
{
	Init();
}


Layout_Single_ImageInfo::~Layout_Single_ImageInfo()
{
}


void Layout_Single_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int width,int height)
{
	GdkPixbuf *thumbnail=GetThumbnail();
	if(transformed)
		g_object_unref(transformed);
	transformed=NULL;

	Layout_Single *l=(Layout_Single *)&layout;

	double xres=(native_xres*100.0)/hscale;
	double yres=(native_yres*100.0)/vscale;

	int w=0,h=0;
	cerr << "rotation " << rotation << endl;
	switch(rotation)
	{
		case PP_ROTATION_AUTO:
		case PP_ROTATION_NONE:
			{
				double pw=image_width*72; pw/=xres;
				double ph=image_height*72; ph/=yres;
				w=int(pw*width/l->pagewidth);
				h=int(ph*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(thumbnail,w,h,GDK_INTERP_NEAREST);
			}
			break;
		case PP_ROTATION_90:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
				w=int(ph*width/l->pagewidth);
				h=int(pw*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
		case PP_ROTATION_180:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_UPSIDEDOWN);
				w=int(pw*width/l->pagewidth);
				h=int(ph*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
		case PP_ROTATION_270:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_CLOCKWISE);
				w=int(ph*width/l->pagewidth);
				h=int(pw*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
	}
	l->GetImageableArea();

	int iw=(l->imageablewidth*width)/l->pagewidth;
	if(w>iw)
		w=iw;

	int ih=(l->imageableheight*height)/l->pageheight;
	if(h>ih)
		h=ih;

	int lm=(l->leftmargin*width)/l->pagewidth;
	int tm=(l->topmargin*height)/l->pageheight;

	gdk_draw_pixbuf(widget->window,NULL,transformed,
		0,0,
		xpos+lm,ypos+tm,
		w,h,
		GDK_RGB_DITHER_NONE,0,0);
}


ImageSource *Layout_Single_ImageInfo::GetImageSource(CMColourDevice target,CMTransformFactory *factory)
{
	ImageSource *is=Layout_ImageInfo::GetImageSource(target,factory);
	is->xres=(native_xres*100)/hscale;
	is->yres=(native_yres*100)/vscale;
	return(is);
}


Layout_Single::Layout_Single(PhotoPrint_State &state,Layout *oldlayout)
	: Layout(state,oldlayout), PageExtent()
{
}


Layout_Single::~Layout_Single()
{
}


void Layout_Single::Reflow()
{
	int page=0;
	Layout_Single_ImageInfo *ii=(Layout_Single_ImageInfo *)FirstImage();
	while(ii)
	{
		ii->page=page;
		ii=(Layout_Single_ImageInfo *)NextImage();
		++page;
	}
	if(page<1)
		page=1;
	pages=page;
	if(currentpage>=page)
		currentpage=page-1;
}


int Layout_Single::AddImage(const char *filename,bool allowcropping,PP_ROTATION rotation)
{
	int page=pages+1;
	for(int i=0;i<=pages;++i)
	{
		if(!ImageAt(i))
		{
			cerr << "No image found at page " << i << endl;
			page=i;
			i=pages;
		}
	}
	cerr << "Adding image to page " << page << endl;
	Layout_Single_ImageInfo *ii=NULL;
	try
	{
		ii=new Layout_Single_ImageInfo(*this,filename,page,false,rotation);
	}
	catch(const char *msg)
	{
		ErrorMessage_Dialog(msg);
	}
	if(ii)
	{
		imagelist=g_list_append(imagelist,ii);

		if(page>=pages)
			++pages;
	}
	return(page);
}


void Layout_Single::CopyImage(Layout_ImageInfo *ii)
{
	int page=pages+1;
	for(int i=0;i<=pages;++i)
	{
		if(!ImageAt(i))
		{
			cerr << "No image found at page " << i << endl;
			page=i;
			i=pages;
		}
	}
	ii=new Layout_Single_ImageInfo(*this,ii,page);
	imagelist=g_list_append(imagelist,ii);
	if(page>=pages)
		++pages;
}


ImageSource *Layout_Single::GetImageSource(int page,CMColourDevice target,CMTransformFactory *factory,int res)
{
	ImageSource *result=NULL;
	try
	{
		if(imagelist)
		{
//			GetDefaultRGBTransform();
//			GetDefaultCMYKTransform();

			Layout_Single_ImageInfo *ii=(Layout_Single_ImageInfo *)ImageAt(page);
			if(ii)
			{
				ImageSource *is=ii->GetImageSource(target,factory);
				switch(ii->rotation)
				{
					case PP_ROTATION_90:
						is=new ImageSource_Rotate(is,90);
						break;
					case PP_ROTATION_180:
						is=new ImageSource_Rotate(is,180);
						break;
					case PP_ROTATION_270:
						is=new ImageSource_Rotate(is,270);
						break;
					default:
						break;
				}
				xoffset=leftmargin;
				yoffset=rightmargin;
				GetImageableArea();
				int iw=int((imageablewidth*is->xres)/72.0);
				int ih=int((imageableheight*is->yres)/72.0);
				if((iw<is->width) || (ih<is->height))
				{
					is=new ImageSource_Crop(is,0,0,iw,ih);
				}
				return(is);
			}
		}
	}
	catch (const char *msg)
	{
		ErrorMessage_Dialog(msg);
	}
	return(result);
}


void Layout_Single::SetPageExtent(PageExtent &pe)
{
	pe.GetImageableArea();
	pagewidth=pe.pagewidth;
	pageheight=pe.pageheight;
	leftmargin=pe.leftmargin;
	rightmargin=pe.rightmargin;
	topmargin=pe.topmargin;
	bottommargin=pe.bottommargin;
	GetImageableArea();
}


void Layout_Single::DBToLayout(LayoutDB &db)
{
	Layout::DBToLayout(db);

	SetPageExtent(state.printer);
	SetMargins(db.singledb.FindInt("LeftMargin"),db.singledb.FindInt("RightMargin"),
		db.singledb.FindInt("TopMargin"),db.singledb.FindInt("BottomMargin"));
}


void Layout_Single::LayoutToDB(LayoutDB &db)
{
	Layout::LayoutToDB(db);

	db.singledb.SetInt("LeftMargin",leftmargin);
	db.singledb.SetInt("RightMargin",rightmargin);
	db.singledb.SetInt("TopMargin",topmargin);
	db.singledb.SetInt("BottomMargin",bottommargin);
}


Layout_Single_ImageInfo *Layout_Single::ImageAt(int page)
{
	Layout_Single_ImageInfo *result=NULL;
	Layout_ImageInfo *ii=FirstImage();
	while(ii)
	{
		Layout_Single_ImageInfo *nii=(Layout_Single_ImageInfo *)ii;
		if(nii->page==page)
		{
			result=nii;
		}
		ii=NextImage();
	}
	return(result);
}


GtkWidget *Layout_Single::CreateWidget()
{
	return(pp_layout_single_new(&state));
}


void Layout_Single::RefreshWidget(GtkWidget *widget)
{
	pp_layout_single_refresh(PP_LAYOUT_SINGLE(widget));
}


const char *Layout_Single::GetType()
{
	return("Single");
}


void Layout_Single::UpdatePageSize()
{
	SetPageExtent(state.printer);
}

void (*Layout_Single::SetUnitFunc())(GtkWidget *wid,enum Units unit)
{
	return(pp_layout_single_set_unit);
}


int Layout_Single::GetCapabilities()
{
	return(PPLAYOUT_ROTATE|PPLAYOUT_MASK|PPLAYOUT_PROFILE);
}

==> photoprint_0.3.5.orig.tar.gz: photoprint-0.3.5/layout_single.cpp <==
/*
 * layout_single.cpp - Has responsibility for tracking the list of images and their layout.
 * Also has responsibility for building the ImageSource stack at print time.
 *
 * Copyright (c) 2004 by Alastair M. Robinson
 * Distributed under the terms of the GNU General Public License -
 * see the file named "COPYING" for more details.
 *
 * 2004-12-24: Now opens the imagesource when placing an image, since we need both the
 *             natural size of the image (which the thumbnail code could provide)
 *             and the resolution of the image (which the thumbnail code can't provide).
 *             Getting the actual imagesource (with CMS transform) from the ImageInfo
 *             is now delegated to the superclasses.
 *
 * 2005-08-21: Added allowcropping and allowrotation parameters to allow for the menu
 *             generalisation.  FIXME: Make sure rotation behaves sanely.
 */

#include <iostream>
#include <string.h>

#include "support/generaldialogs.h"
#include "dialogs.h"
#include "pixbufthumbnail/egg-pixbuf-thumbnail.h"
#include "support/rotatepixbuf.h"
#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_crop.h"
#include "imagesource/imagesource_rotate.h"
#include "photoprint_state.h"
#include "pp_layout_single.h"

#include "layout_single.h"

using namespace std;


ConfigTemplate Layout_SingleDB::Template[]=
{
	ConfigTemplate("LeftMargin",10),
	ConfigTemplate("RightMargin",10),
	ConfigTemplate("TopMargin",10),
	ConfigTemplate("BottomMargin",10),
	ConfigTemplate()
};


void Layout_Single_ImageInfo::Init()
{
	Layout_Single *l=(Layout_Single *)&layout;
	ImageSource *is=ISLoadImage(filename);
	image_width=is->width;
	image_height=is->height;
	native_xres=is->xres;
	native_yres=is->yres;
	delete is;
	
	l->GetImageableArea();
	int sw=int((image_width*72.0)/native_xres);
	int sh=int((image_height*72.0)/native_yres);
	int mm=(l->imageablewidth>l->imageableheight);
	mm^=image_width>image_height;
	if(mm)
	{
		rotation=PP_ROTATION_90;
		sw=int((image_height*72.0)/native_yres);
		sh=int((image_width*72.0)/native_xres);
	}
	else
		rotation=PP_ROTATION_NONE;
	int s=100;
	int s2=100;
	if(sw>l->imageablewidth)
		s=(100*l->imageablewidth)/sw;
	if(sh>l->imageableheight)
		s2=(100*l->imageableheight)/sh;

	if(s2<s)
		s=s2;
	
	hscale=s;
	vscale=s;
}


Layout_Single_ImageInfo::Layout_Single_ImageInfo(Layout_Single &layout, const char *filename,int page,bool allowcropping,PP_ROTATION rotation)
	: Layout_ImageInfo((Layout &)layout,filename,page,allowcropping,rotation), hscale(100), vscale(100)
{
	Init();
}


Layout_Single_ImageInfo::Layout_Single_ImageInfo(Layout_Single &layout, Layout_ImageInfo *ii,int page,bool allowcropping,PP_ROTATION rotation)
	: Layout_ImageInfo((Layout &)layout,ii,page,allowcropping,rotation), hscale(100), vscale(100)
{
	Init();
}


Layout_Single_ImageInfo::~Layout_Single_ImageInfo()
{
}


void Layout_Single_ImageInfo::DrawThumbnail(GtkWidget *widget,int xpos,int ypos,int width,int height)
{
	GdkPixbuf *thumbnail=GetThumbnail();
	if(transformed)
		g_object_unref(transformed);
	transformed=NULL;

	Layout_Single *l=(Layout_Single *)&layout;

	double xres=(native_xres*100.0)/hscale;
	double yres=(native_yres*100.0)/vscale;

	int w=0,h=0;
	cerr << "rotation " << rotation << endl;
	switch(rotation)
	{
		case PP_ROTATION_AUTO:
		case PP_ROTATION_NONE:
			{
				double pw=image_width*72; pw/=xres;
				double ph=image_height*72; ph/=yres;
				w=int(pw*width/l->pagewidth);
				h=int(ph*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(thumbnail,w,h,GDK_INTERP_NEAREST);
			}
			break;
		case PP_ROTATION_90:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
				w=int(ph*width/l->pagewidth);
				h=int(pw*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
		case PP_ROTATION_180:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_UPSIDEDOWN);
				w=int(pw*width/l->pagewidth);
				h=int(ph*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
		case PP_ROTATION_270:
			{
				double pw=image_width*72; pw/=yres;
				double ph=image_height*72; ph/=xres;
				GdkPixbuf *tmp=gdk_pixbuf_rotate_simple(thumbnail,GDK_PIXBUF_ROTATE_CLOCKWISE);
				w=int(ph*width/l->pagewidth);
				h=int(pw*height/l->pageheight);
				transformed=gdk_pixbuf_scale_simple(tmp,w,h,GDK_INTERP_NEAREST);
				g_object_unref(G_OBJECT(tmp));
			}
			break;
	}
	l->GetImageableArea();

	int iw=(l->imageablewidth*width)/l->pagewidth;
	if(w>iw)
		w=iw;

	int ih=(l->imageableheight*height)/l->pageheight;
	if(h>ih)
		h=ih;

	int lm=(l->leftmargin*width)/l->pagewidth;
	int tm=(l->topmargin*height)/l->pageheight;

	gdk_draw_pixbuf(widget->window,NULL,transformed,
		0,0,
		xpos+lm,ypos+tm,
		w,h,
		GDK_RGB_DITHER_NONE,0,0);
}


ImageSource *Layout_Single_ImageInfo::GetImageSource(CMColourDevice target,CMTransformFactory *factory)
{
	ImageSource *is=Layout_ImageInfo::GetImageSource(target,factory);
	is->xres=(native_xres*100)/hscale;
	is->yres=(native_yres*100)/vscale;
	return(is);
}


Layout_Single::Layout_Single(PhotoPrint_State &state,Layout *oldlayout)
	: Layout(state,oldlayout)
{
}


Layout_Single::~Layout_Single()
{
}


void Layout_Single::Reflow()
{
	int page=0;
	Layout_Single_ImageInfo *ii=(Layout_Single_ImageInfo *)FirstImage();
	while(ii)
	{
		ii->page=page;
		ii=(Layout_Single_ImageInfo *)NextImage();
		++page;
	}
	if(page<1)
		page=1;
	pages=page;
	if(currentpage>=page)
		currentpage=page-1;
}


int Layout_Single::AddImage(const char *filename,bool allowcropping,PP_ROTATION rotation)
{
	int page=pages+1;
	for(int i=0;i<=pages;++i)
	{
		if(!ImageAt(i))
		{
			cerr << "No image found at page " << i << endl;
			page=i;
			i=pages;
		}
	}
	cerr << "Adding image to page " << page << endl;
	Layout_Single_ImageInfo *ii=NULL;
	try
	{
		ii=new Layout_Single_ImageInfo(*this,filename,page,false,rotation);
	}
	catch(const char *msg)
	{
		ErrorMessage_Dialog(msg);
	}
	if(ii)
	{
		imagelist=g_list_append(imagelist,ii);

		if(page>=pages)
			++pages;
	}
	return(page);
}


void Layout_Single::CopyImage(Layout_ImageInfo *ii)
{
	int page=pages+1;
	for(int i=0;i<=pages;++i)
	{
		if(!ImageAt(i))
		{
			cerr << "No image found at page " << i << endl;
			page=i;
			i=pages;
		}
	}
	ii=new Layout_Single_ImageInfo(*this,ii,page);
	imagelist=g_list_append(imagelist,ii);
	if(page>=pages)
		++pages;
}


ImageSource *Layout_Single::GetImageSource(int page,CMColourDevice target,CMTransformFactory *factory,int res)
{
	ImageSource *result=NULL;
	try
	{
		if(imagelist)
		{
//			GetDefaultRGBTransform();
//			GetDefaultCMYKTransform();

			Layout_Single_ImageInfo *ii=(Layout_Single_ImageInfo *)ImageAt(page);
			if(ii)
			{
				ImageSource *is=ii->GetImageSource(target,factory);
				switch(ii->rotation)
				{
					case PP_ROTATION_90:
						is=new ImageSource_Rotate(is,90);
						break;
					case PP_ROTATION_180:
						is=new ImageSource_Rotate(is,180);
						break;
					case PP_ROTATION_270:
						is=new ImageSource_Rotate(is,270);
						break;
					default:
						break;
				}
				xoffset=leftmargin;
				yoffset=rightmargin;
				GetImageableArea();
				int iw=int((imageablewidth*is->xres)/72.0);
				int ih=int((imageableheight*is->yres)/72.0);
				if((iw<is->width) || (ih<is->height))
				{
					is=new ImageSource_Crop(is,0,0,iw,ih);
				}
				return(is);
			}
		}
	}
	catch (const char *msg)
	{
		ErrorMessage_Dialog(msg);
	}
	return(result);
}


void Layout_Single::SetPageExtent(PageExtent &pe)
{
	pe.GetImageableArea();
	pagewidth=pe.pagewidth;
	pageheight=pe.pageheight;
	leftmargin=pe.leftmargin;
	rightmargin=pe.rightmargin;
	topmargin=pe.topmargin;
	bottommargin=pe.bottommargin;
	GetImageableArea();
}


void Layout_Single::DBToLayout(LayoutDB &db)
{
	Layout::DBToLayout(db);

	SetPageExtent(state.printer);
	SetMargins(db.singledb.FindInt("LeftMargin"),db.singledb.FindInt("RightMargin"),
		db.singledb.FindInt("TopMargin"),db.singledb.FindInt("BottomMargin"));
}


void Layout_Single::LayoutToDB(LayoutDB &db)
{
	Layout::LayoutToDB(db);

	db.singledb.SetInt("LeftMargin",leftmargin);
	db.singledb.SetInt("RightMargin",rightmargin);
	db.singledb.SetInt("TopMargin",topmargin);
	db.singledb.SetInt("BottomMargin",bottommargin);
}


Layout_Single_ImageInfo *Layout_Single::ImageAt(int page)
{
	Layout_Single_ImageInfo *result=NULL;
	Layout_ImageInfo *ii=FirstImage();
	while(ii)
	{
		Layout_Single_ImageInfo *nii=(Layout_Single_ImageInfo *)ii;
		if(nii->page==page)
		{
			result=nii;
		}
		ii=NextImage();
	}
	return(result);
}


GtkWidget *Layout_Single::CreateWidget()
{
	return(pp_layout_single_new(&state));
}


void Layout_Single::RefreshWidget(GtkWidget *widget)
{
	pp_layout_single_refresh(PP_LAYOUT_SINGLE(widget));
}


const char *Layout_Single::GetType()
{
	return("Single");
}


void Layout_Single::UpdatePageSize()
{
	SetPageExtent(state.printer);
}

void (*Layout_Single::SetUnitFunc())(GtkWidget *wid,enum Units unit)
{
	return(pp_layout_single_set_unit);
}


int Layout_Single::GetCapabilities()
{
	return(PPLAYOUT_ROTATE|PPLAYOUT_MASK|PPLAYOUT_EFFECTS|PPLAYOUT_PROFILE);
}
